package ru.dbotthepony.kvector.api.concrete

import ru.dbotthepony.kvector.api.IDoubleMatrix
import ru.dbotthepony.kvector.api.IMatrix
import ru.dbotthepony.kvector.api.ISquareMatrix
import ru.dbotthepony.kvector.api.ITransposable
import ru.dbotthepony.kvector.matrix.*
import ru.dbotthepony.kvector.vector.ndouble.Vector2d
import ru.dbotthepony.kvector.vector.ndouble.Vector3d
import ru.dbotthepony.kvector.vector.ndouble.Vector4d

/**
 * [Matrix2d] and [MutableMatrix2d] implement this
 */
interface IMatrix2d<T : IMatrix2d<T>> : IMatrix<T>, IDoubleMatrix<T>, ITransposable<T> {
	val c00: Double
	val c10: Double
	val c01: Double
	val c11: Double
	val r00: Double
	val r01: Double
	val r10: Double
	val r11: Double
	val a11: Double
	val a21: Double
	val a12: Double
	val a22: Double
	val b11: Double
	val b21: Double
	val b12: Double
	val b22: Double
	val c0: Vector2d
	val c1: Vector2d
	val r0: Vector2d
	val r1: Vector2d
}

/**
 * [Matrix3d] and [MutableMatrix3d] implement this
 */
interface IMatrix3d<T : IMatrix3d<T>> : IMatrix<T>, IDoubleMatrix<T>, ITransposable<T>, ISquareMatrix<T, Vector2d> {
	val c00: Double
	val c10: Double
	val c20: Double
	val c01: Double
	val c11: Double
	val c21: Double
	val c02: Double
	val c12: Double
	val c22: Double
	val r00: Double
	val r01: Double
	val r02: Double
	val r10: Double
	val r11: Double
	val r12: Double
	val r20: Double
	val r21: Double
	val r22: Double
	val a11: Double
	val a21: Double
	val a31: Double
	val a12: Double
	val a22: Double
	val a32: Double
	val a13: Double
	val a23: Double
	val a33: Double
	val b11: Double
	val b21: Double
	val b31: Double
	val b12: Double
	val b22: Double
	val b32: Double
	val b13: Double
	val b23: Double
	val b33: Double
	val c0: Vector3d
	val c1: Vector3d
	val c2: Vector3d
	val r0: Vector3d
	val r1: Vector3d
	val r2: Vector3d

	override val translation: Vector2d get() = Vector2d(r02, r12)
}

/**
 * [Matrix4d] and [MutableMatrix4d] implement this
 */
interface IMatrix4d<T : IMatrix4d<T>> : IMatrix<T>, IDoubleMatrix<T>, ITransposable<T>, ISquareMatrix<T, Vector3d> {
	val c00: Double
	val c10: Double
	val c20: Double
	val c30: Double
	val c01: Double
	val c11: Double
	val c21: Double
	val c31: Double
	val c02: Double
	val c12: Double
	val c22: Double
	val c32: Double
	val c03: Double
	val c13: Double
	val c23: Double
	val c33: Double
	val r00: Double
	val r01: Double
	val r02: Double
	val r03: Double
	val r10: Double
	val r11: Double
	val r12: Double
	val r13: Double
	val r20: Double
	val r21: Double
	val r22: Double
	val r23: Double
	val r30: Double
	val r31: Double
	val r32: Double
	val r33: Double
	val a11: Double
	val a21: Double
	val a31: Double
	val a41: Double
	val a12: Double
	val a22: Double
	val a32: Double
	val a42: Double
	val a13: Double
	val a23: Double
	val a33: Double
	val a43: Double
	val a14: Double
	val a24: Double
	val a34: Double
	val a44: Double
	val b11: Double
	val b21: Double
	val b31: Double
	val b41: Double
	val b12: Double
	val b22: Double
	val b32: Double
	val b42: Double
	val b13: Double
	val b23: Double
	val b33: Double
	val b43: Double
	val b14: Double
	val b24: Double
	val b34: Double
	val b44: Double
	val c0: Vector4d
	val c1: Vector4d
	val c2: Vector4d
	val c3: Vector4d
	val r0: Vector4d
	val r1: Vector4d
	val r2: Vector4d
	val r3: Vector4d

	override val translation: Vector3d get() = Vector3d(r03, r13, r23)
}
